---
title: "Tensorboard"
sub_link: "tensorboard"
meta_title: "Hyperparams Optimization in Polyaxon - Core Concepts"
meta_description: "A Tensorboard is a job running to visualize the metrics of an experiment,
the experiments of a group, or of a project."
tags:
    - concepts
    - polyaxon
    - experimentation
    - tensorboard
    - visualization
    - architecture
sidebar: "concepts"
---

![notebooks](../../content/images/concepts/dashboard/tensorboards.png)

Polyaxon allows users to run tensorboard jobs on project, experiment, and experiment group level, these jobs are subject to the same permissions of the project they belong to.


[Tensorboard](https://www.tensorflow.org/programmers_guide/summaries_and_tensorboard) is a visualization tool for Tensorflow projects.
Tensorboard can help visualize the Tensorflow computation graph and plot quantitative metrics about your run.

## Start TensorBoard

We assume that you have already a [project](/concepts/projects/) created and initialized, and code uploaded.

To start tensorboard for a project, run the following command in your terminal

```bash
$ polyaxon tensorboard start


Tensorboard is being deployed for project `mnist`

It may take some time before you can access the dashboard.

Your Tensorboard will be available on:

    http://192.168.64.6:31122/tensorboard/root/quick-start/
```

This will start tensorboard with the default options,
if you wish to start tensorboard with a different Tensorflow image, or custom resources,
you need to create a polyaxonfile containing:

 * [version](/references/polyaxonfile-yaml-specification/version/)
 * [run](/references/polyaxonfile-yaml-specification/run/)

For example to start tensorboard with Tensorflow 1.6, you need to define a new polyaxonfile_tensorboard.yml with the following options:


```yaml
version: 1

kind: tensorboard

environment:
  resources:
    cpu:
      requests: 2
      limits: 4
    memory:
      requests: 512
      limits: 2048

build:
  image: tensorflow/tensorflow:1.6.0-py3
```

You can also start a tensorboard for a specific experiment, or for aggregating metrics from all experiments in a group.


```bash
$ polyaxon tensorboard -xp 13 start


Tensorboard is being deployed for experiment 13

It may take some time before you can access the dashboard.

Your Tensorboard will be available on:

    http://192.168.64.6:31122/tensorboard/root/quick-start/experiments/13/
```
```bash
$ polyaxon tensorboard -g 3 start


Tensorboard is being deployed for experiment group 3

It may take some time before you can access the dashboard.

Your Tensorboard will be available on:

    http://192.168.64.6:31122/tensorboard/root/quick-start/groups/13/
```

### Change the default tensorboard

You can change the default image used to for deploying tensorboards, this way you won't need polyaxonfiles, 
to do that you need to update your deployment config file:

```yaml
tensorboardDockerImage:  image:tag
``` 

And then upgrade your deployment using the new config file.

### Stop tensorboard

To stop tensorboard, run the following command in your terminal

For project tensorboard

```bash
$ polyaxon tensorboard stop
```

For experiment tensorboard

```bash
$ polyaxon tensorboard -xp 13 stop
```

For experiment group tensorboard

```bash
$ polyaxon tensorboard -g 3 stop
```

> For more details about this command please run `polyaxon tensorboard --help`, 
or check the [command reference](/references/polyaxon-cli/tensorboard/)
